package org.eclipse.jdt.internal.compiler.tool;

import java.nio.charset.Charset;
import org.eclipse.jdt.internal.compiler.problem.DefaultProblem;
import org.eclipse.jdt.internal.compiler.batch.FileSystem;
import org.eclipse.jdt.internal.compiler.batch.ClasspathJsr199;
import javax.tools.StandardJavaFileManager;
import java.io.OutputStream;
import org.eclipse.jdt.internal.compiler.ClassFile;
import java.io.BufferedOutputStream;
import javax.tools.FileObject;
import org.eclipse.jdt.internal.compiler.util.Messages;
import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.util.SuffixConstants;
import javax.tools.StandardLocation;
import org.eclipse.jdt.internal.compiler.CompilationResult;
import org.eclipse.jdt.core.compiler.CompilationProgress;
import java.util.Map;
import java.io.File;
import java.util.Locale;
import javax.tools.Diagnostic;
import org.eclipse.jdt.core.compiler.CategorizedProblem;
import org.eclipse.jdt.internal.compiler.problem.DefaultProblemFactory;
import org.eclipse.jdt.internal.compiler.IProblemFactory;
import org.eclipse.jdt.internal.compiler.IErrorHandlingPolicy;
import java.util.Iterator;
import org.eclipse.jdt.internal.compiler.problem.AbortCompilationUnit;
import java.util.ArrayList;
import java.io.IOException;
import java.io.PrintWriter;
import javax.tools.DiagnosticListener;
import javax.annotation.processing.Processor;
import javax.tools.JavaFileManager;
import javax.tools.JavaFileObject;
import java.util.HashMap;
import org.eclipse.jdt.internal.compiler.batch.CompilationUnit;
import org.eclipse.jdt.internal.compiler.batch.Main;

public class EclipseCompilerImpl extends Main
{
    private static final CompilationUnit[] NO_UNITS;
    private HashMap<CompilationUnit, JavaFileObject> javaFileObjectMap;
    Iterable<? extends JavaFileObject> compilationUnits;
    public JavaFileManager fileManager;
    protected Processor[] processors;
    public DiagnosticListener<? super JavaFileObject> diagnosticListener;
    
    static {
        NO_UNITS = new CompilationUnit[0];
    }
    
    public EclipseCompilerImpl(PrintWriter out, PrintWriter err, boolean systemExitWhenFinished) {
        super(out, err, systemExitWhenFinished, null, null);
    }
    
    public boolean call() {
        try {
            if (this.proceed) {
                this.globalProblemsCount = 0;
                this.globalErrorsCount = 0;
                this.globalWarningsCount = 0;
                this.globalTasksCount = 0;
                this.exportedClassFilesCounter = 0;
                this.performCompilation();
            }
        }
        catch (IllegalArgumentException e) {
            this.logger.logException(e);
            if (this.systemExitWhenFinished) {
                this.cleanup();
                System.exit(-1);
            }
            return false;
        }
        catch (RuntimeException e2) {
            this.logger.logException(e2);
            return false;
        }
        finally {
            this.cleanup();
        }
        return this.globalErrorsCount == 0;
    }
    
    private void cleanup() {
        this.logger.flush();
        this.logger.close();
        this.processors = null;
        try {
            if (this.fileManager != null) {
                this.fileManager.flush();
            }
        }
        catch (IOException ex) {}
    }
    
    public CompilationUnit[] getCompilationUnits() {
        if (this.compilationUnits == null) {
            return EclipseCompilerImpl.NO_UNITS;
        }
        ArrayList<CompilationUnit> units = new ArrayList();
        for (JavaFileObject javaFileObject : this.compilationUnits) {
            if (javaFileObject.getKind() != JavaFileObject.Kind.SOURCE) {
                throw new IllegalArgumentException();
            }
            final String name = javaFileObject.getName();
            name = name.replace('\\', '/');
            CompilationUnit compilationUnit = new CompilationUnit(null, name, null) {
                public char[] getContents() {
                    try {
                        return javaFileObject.getCharContent(true).toString().toCharArray();
                    }
                    catch (IOException e) {
                        e.printStackTrace();
                        throw new AbortCompilationUnit(null, e, null);
                    }
                }
            };
            units.add(compilationUnit);
            this.javaFileObjectMap.put(compilationUnit, javaFileObject);
        }
        CompilationUnit[] result = new CompilationUnit[units.size()];
        units.toArray(result);
        return result;
    }
    
    public IErrorHandlingPolicy getHandlingPolicy() {
        return new IErrorHandlingPolicy() {
            public boolean proceedOnErrors() {
                return false;
            }
            
            public boolean stopOnFirstError() {
                return false;
            }
            
            public boolean ignoreAllErrors() {
                return false;
            }
        };
    }
    
    public IProblemFactory getProblemFactory() {
        return new DefaultProblemFactory() {
            public CategorizedProblem createProblem(char[] originatingFileName, int problemId, String[] problemArguments, String[] messageArguments, int severity, int startPosition, int endPosition, int lineNumber, int columnNumber) {
                DiagnosticListener<? super JavaFileObject> diagListener = EclipseCompilerImpl.this.diagnosticListener;
                if (diagListener != null) {
                    diagListener.report(new Diagnostic<JavaFileObject>() {
                        @Override
                        public String getCode() {
                            return Integer.toString(problemId);
                        }
                        
                        @Override
                        public long getColumnNumber() {
                            return (long)columnNumber;
                        }
                        
                        @Override
                        public long getEndPosition() {
                            return (long)endPosition;
                        }
                        
                        @Override
                        public Diagnostic.Kind getKind() {
                            if ((severity & 0x1) != 0x0) {
                                return Diagnostic.Kind.ERROR;
                            }
                            if ((severity & 0x20) != 0x0) {
                                return Diagnostic.Kind.WARNING;
                            }
                            if (false) {
                                return Diagnostic.Kind.MANDATORY_WARNING;
                            }
                            return Diagnostic.Kind.OTHER;
                        }
                        
                        @Override
                        public long getLineNumber() {
                            return (long)lineNumber;
                        }
                        
                        @Override
                        public String getMessage(Locale locale) {
                            if (locale != null) {
                                EclipseCompilerImpl.3.this.setLocale(locale);
                            }
                            return EclipseCompilerImpl.3.this.getLocalizedMessage(problemId, problemArguments);
                        }
                        
                        @Override
                        public long getPosition() {
                            return (long)startPosition;
                        }
                        
                        @Override
                        public JavaFileObject getSource() {
                            File f = new File(new String(originatingFileName));
                            if (f.exists()) {
                                return new EclipseFileObject(null, f.toURI(), JavaFileObject.Kind.SOURCE, null);
                            }
                            return null;
                        }
                        
                        @Override
                        public long getStartPosition() {
                            return (long)startPosition;
                        }
                    });
                }
                return super.createProblem(originatingFileName, problemId, problemArguments, messageArguments, severity, startPosition, endPosition, lineNumber, columnNumber);
            }
            
            public CategorizedProblem createProblem(char[] originatingFileName, int problemId, String[] problemArguments, int elaborationID, String[] messageArguments, int severity, int startPosition, int endPosition, int lineNumber, int columnNumber) {
                DiagnosticListener<? super JavaFileObject> diagListener = EclipseCompilerImpl.this.diagnosticListener;
                if (diagListener != null) {
                    diagListener.report(new Diagnostic<JavaFileObject>() {
                        @Override
                        public String getCode() {
                            return Integer.toString(problemId);
                        }
                        
                        @Override
                        public long getColumnNumber() {
                            return (long)columnNumber;
                        }
                        
                        @Override
                        public long getEndPosition() {
                            return (long)endPosition;
                        }
                        
                        @Override
                        public Diagnostic.Kind getKind() {
                            if ((severity & 0x1) != 0x0) {
                                return Diagnostic.Kind.ERROR;
                            }
                            if ((severity & 0x400) != 0x0) {
                                return Diagnostic.Kind.NOTE;
                            }
                            if ((severity & 0x20) != 0x0) {
                                return Diagnostic.Kind.WARNING;
                            }
                            if (false) {
                                return Diagnostic.Kind.MANDATORY_WARNING;
                            }
                            return Diagnostic.Kind.OTHER;
                        }
                        
                        @Override
                        public long getLineNumber() {
                            return (long)lineNumber;
                        }
                        
                        @Override
                        public String getMessage(Locale locale) {
                            if (locale != null) {
                                EclipseCompilerImpl.3.this.setLocale(locale);
                            }
                            return EclipseCompilerImpl.3.this.getLocalizedMessage(problemId, problemArguments);
                        }
                        
                        @Override
                        public long getPosition() {
                            return (long)startPosition;
                        }
                        
                        @Override
                        public JavaFileObject getSource() {
                            File f = new File(new String(originatingFileName));
                            if (f.exists()) {
                                return new EclipseFileObject(null, f.toURI(), JavaFileObject.Kind.SOURCE, null);
                            }
                            return null;
                        }
                        
                        @Override
                        public long getStartPosition() {
                            return (long)startPosition;
                        }
                    });
                }
                return super.createProblem(originatingFileName, problemId, problemArguments, elaborationID, messageArguments, severity, startPosition, endPosition, lineNumber, columnNumber);
            }
        };
    }
    
    protected void initialize(PrintWriter outWriter, PrintWriter errWriter, boolean systemExit, Map<String, String> customDefaultOptions, CompilationProgress compilationProgress) {
        super.initialize(outWriter, errWriter, systemExit, customDefaultOptions, compilationProgress);
        this.javaFileObjectMap = new HashMap();
    }
    
    protected void initializeAnnotationProcessorManager() {
        super.initializeAnnotationProcessorManager();
        if (this.batchCompiler.annotationProcessorManager != null && this.processors != null) {
            this.batchCompiler.annotationProcessorManager.setProcessors(this.processors);
        }
        else if (this.processors != null) {
            throw new UnsupportedOperationException("Cannot handle annotation processing");
        }
    }
    
    public void outputClassFiles(CompilationResult unitResult) {
        if (unitResult != null && (!unitResult.hasErrors() || this.proceedOnError)) {
            ClassFile[] classFiles = unitResult.getClassFiles();
            boolean generateClasspathStructure = this.fileManager.hasLocation(StandardLocation.CLASS_OUTPUT);
            String currentDestinationPath = this.destinationPath;
            File outputLocation = null;
            if (currentDestinationPath != null) {
                outputLocation = new File(currentDestinationPath);
                outputLocation.mkdirs();
            }
            for (int i = 0, fileCount = classFiles.length; i < fileCount; ++this.exportedClassFilesCounter, ++i) {
                ClassFile classFile = classFiles[i];
                char[] filename = classFile.fileName();
                int length = filename.length;
                char[] relativeName = new char[length + 6];
                System.arraycopy(filename, 0, relativeName, 0, length);
                System.arraycopy(SuffixConstants.SUFFIX_class, 0, relativeName, length, 6);
                CharOperation.replace(relativeName, '/', File.separatorChar);
                String relativeStringName = new String(relativeName);
                if (this.compilerOptions.verbose) {
                    this.out.println(Messages.bind(Messages.compilation_write, new String[] { String.valueOf(this.exportedClassFilesCounter + 1), relativeStringName }));
                }
                try {
                    JavaFileObject javaFileForOutput = this.fileManager.getJavaFileForOutput(StandardLocation.CLASS_OUTPUT, new String(filename), JavaFileObject.Kind.CLASS, (FileObject)this.javaFileObjectMap.get(unitResult.compilationUnit));
                    if (generateClasspathStructure) {
                        if (currentDestinationPath != null) {
                            int index2 = CharOperation.lastIndexOf(File.separatorChar, relativeName);
                            if (index2 != -1) {
                                File currentFolder = new File(currentDestinationPath, relativeStringName.substring(0, index2));
                                currentFolder.mkdirs();
                            }
                        }
                        else {
                            String path = javaFileForOutput.toUri().getPath();
                            int index = path.lastIndexOf(47);
                            if (index != -1) {
                                File file = new File(path.substring(0, index));
                                file.mkdirs();
                            }
                        }
                    }
                    Throwable t = null;
                    try {
                        OutputStream openOutputStream = javaFileForOutput.openOutputStream();
                        try {
                            BufferedOutputStream bufferedOutputStream = new BufferedOutputStream(openOutputStream);
                            try {
                                bufferedOutputStream.write(classFile.header, 0, classFile.headerOffset);
                                bufferedOutputStream.write(classFile.contents, 0, classFile.contentsOffset);
                                bufferedOutputStream.flush();
                            }
                            finally {
                                if (bufferedOutputStream != null) {
                                    bufferedOutputStream.close();
                                }
                            }
                            if (openOutputStream != null) {
                                openOutputStream.close();
                            }
                        }
                        finally {
                            if (t == null) {
                                Throwable exception;
                                t = exception;
                            }
                            else {
                                Throwable exception;
                                if (t != exception) {
                                    t.addSuppressed(exception);
                                }
                            }
                            if (openOutputStream != null) {
                                openOutputStream.close();
                            }
                        }
                    }
                    finally {
                        if (t == null) {
                            Throwable exception2;
                            t = exception2;
                        }
                        else {
                            Throwable exception2;
                            if (t != exception2) {
                                t.addSuppressed(exception2);
                            }
                        }
                    }
                }
                catch (IOException e) {
                    this.logger.logNoClassFileCreated(currentDestinationPath, relativeStringName, e);
                }
                this.logger.logClassFile(generateClasspathStructure, currentDestinationPath, relativeStringName);
            }
            this.batchCompiler.lookupEnvironment.releaseClassFiles(classFiles);
        }
    }
    
    protected void setPaths(ArrayList bootclasspaths, String sourcepathClasspathArg, ArrayList sourcepathClasspaths, ArrayList classpaths, ArrayList extdirsClasspaths, ArrayList endorsedDirClasspaths, String customEncoding) {
        ArrayList<FileSystem.Classpath> fileSystemClasspaths = new ArrayList();
        EclipseFileManager eclipseJavaFileManager = null;
        StandardJavaFileManager standardJavaFileManager = null;
        JavaFileManager javaFileManager = null;
        boolean havePlatformPaths = false;
        boolean haveClassPaths = false;
        if ((this.fileManager instanceof EclipseFileManager)) {
            eclipseJavaFileManager = (EclipseFileManager)this.fileManager;
        }
        if ((this.fileManager instanceof StandardJavaFileManager)) {
            standardJavaFileManager = (StandardJavaFileManager)this.fileManager;
        }
        javaFileManager = this.fileManager;
        if (eclipseJavaFileManager != null && (eclipseJavaFileManager.flags & 0x4) == 0x0 && (eclipseJavaFileManager.flags & 0x2) != 0x0) {
            fileSystemClasspaths.addAll(this.handleEndorseddirs(null));
        }
        Iterable<? extends File> location = null;
        if (standardJavaFileManager != null) {
            location = standardJavaFileManager.getLocation(StandardLocation.PLATFORM_CLASS_PATH);
            if (location != null) {
                for (File file : location) {
                    FileSystem.Classpath classpath2 = FileSystem.getClasspath(file.getAbsolutePath(), null, null, this.options);
                    if (classpath2 != null) {
                        fileSystemClasspaths.add(classpath2);
                        havePlatformPaths = true;
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        else if (javaFileManager != null) {
            FileSystem.Classpath classpath = new ClasspathJsr199(this.fileManager, StandardLocation.PLATFORM_CLASS_PATH);
            fileSystemClasspaths.add(classpath);
            havePlatformPaths = true;
        }
        if (eclipseJavaFileManager != null && (eclipseJavaFileManager.flags & 0x1) == 0x0 && (eclipseJavaFileManager.flags & 0x2) != 0x0) {
            fileSystemClasspaths.addAll(this.handleExtdirs(null));
        }
        if (standardJavaFileManager != null) {
            location = standardJavaFileManager.getLocation(StandardLocation.SOURCE_PATH);
            if (location != null) {
                for (File file : location) {
                    FileSystem.Classpath classpath2 = FileSystem.getClasspath(file.getAbsolutePath(), null, null, this.options);
                    if (classpath2 != null) {
                        fileSystemClasspaths.add(classpath2);
                    }
                    else {
                        continue;
                    }
                }
            }
            location = standardJavaFileManager.getLocation(StandardLocation.CLASS_PATH);
            if (location != null) {
                for (File file : location) {
                    FileSystem.Classpath classpath2 = FileSystem.getClasspath(file.getAbsolutePath(), null, null, this.options);
                    if (classpath2 != null) {
                        fileSystemClasspaths.add(classpath2);
                        haveClassPaths = true;
                    }
                    else {
                        continue;
                    }
                }
            }
        }
        else if (javaFileManager != null) {
            FileSystem.Classpath classpath = null;
            if (this.fileManager.hasLocation(StandardLocation.SOURCE_PATH)) {
                classpath = new ClasspathJsr199(this.fileManager, StandardLocation.SOURCE_PATH);
                fileSystemClasspaths.add(classpath);
            }
            classpath = new ClasspathJsr199(this.fileManager, StandardLocation.CLASS_PATH);
            fileSystemClasspaths.add(classpath);
            haveClassPaths = true;
        }
        if (this.checkedClasspaths == null) {
            if (!havePlatformPaths) {
                fileSystemClasspaths.addAll(this.handleBootclasspath(null, null));
            }
            if (!haveClassPaths) {
                fileSystemClasspaths.addAll(this.handleClasspath(null, null));
            }
        }
        fileSystemClasspaths = FileSystem.ClasspathNormalizer.normalize(fileSystemClasspaths);
        int size = fileSystemClasspaths.size();
        if (size != 0) {
            this.checkedClasspaths = new FileSystem.Classpath[size];
            int i = 0;
            for (FileSystem.Classpath classpath2 : fileSystemClasspaths) {
                this.checkedClasspaths[i++] = classpath2;
            }
        }
    }
    
    protected void loggingExtraProblems() {
        super.loggingExtraProblems();
        for (CategorizedProblem problem : this.extraProblems) {
            if (this.diagnosticListener != null) {
                this.diagnosticListener.report(new Diagnostic<JavaFileObject>() {
                    @Override
                    public String getCode() {
                        return null;
                    }
                    
                    @Override
                    public long getColumnNumber() {
                        if ((problem instanceof DefaultProblem)) {
                            return (long)((DefaultProblem)problem).column;
                        }
                        return -1L;
                    }
                    
                    @Override
                    public long getEndPosition() {
                        if ((problem instanceof DefaultProblem)) {
                            return (long)((DefaultProblem)problem).getSourceEnd();
                        }
                        return -1L;
                    }
                    
                    @Override
                    public Diagnostic.Kind getKind() {
                        if (problem.isError()) {
                            return Diagnostic.Kind.ERROR;
                        }
                        if (problem.isWarning()) {
                            return Diagnostic.Kind.WARNING;
                        }
                        if (problem instanceof DefaultProblem && ((DefaultProblem)problem).isInfo()) {
                            return Diagnostic.Kind.NOTE;
                        }
                        return Diagnostic.Kind.OTHER;
                    }
                    
                    @Override
                    public long getLineNumber() {
                        if ((problem instanceof DefaultProblem)) {
                            return (long)((DefaultProblem)problem).getSourceLineNumber();
                        }
                        return -1L;
                    }
                    
                    @Override
                    public String getMessage(Locale locale) {
                        return problem.getMessage();
                    }
                    
                    @Override
                    public long getPosition() {
                        if ((problem instanceof DefaultProblem)) {
                            return (long)((DefaultProblem)problem).getSourceStart();
                        }
                        return -1L;
                    }
                    
                    @Override
                    public JavaFileObject getSource() {
                        if ((problem instanceof DefaultProblem)) {
                            File f = new File(new String(((DefaultProblem)problem).getOriginatingFileName()));
                            if (f.exists()) {
                                Charset charset = (EclipseCompilerImpl.this.fileManager instanceof EclipseFileManager) ? ((EclipseFileManager)EclipseCompilerImpl.this.fileManager).charset : Charset.defaultCharset();
                                return new EclipseFileObject(null, f.toURI(), JavaFileObject.Kind.SOURCE, charset);
                            }
                            return null;
                        }
                        else {
                            return null;
                        }
                    }
                    
                    @Override
                    public long getStartPosition() {
                        return this.getPosition();
                    }
                });
            }
            else {
                continue;
            }
        }
    }
}
